@@ -12,7 +12,7 @@
 /*	and reports TLS-related information about the server. With SMTP, the
 /*	destination is a domainname; with LMTP it is either a domainname
 /*	prefixed with \fBinet:\fR or a pathname prefixed with \fBunix:\fR.  If
-/*	Postfix is built without TLS support, the resulting \fBposttls-finger\fR(1)
+/*	Postfix is built without TLS support, the resulting posttls-finger
 /*	program has very limited functionality, and only the \fB-a\fR, \fB-c\fR,
 /*	\fB-h\fR, \fB-o\fR, \fB-S\fR, \fB-t\fR, \fB-T\fR and \fB-v\fR options
 /*	are available.
@@ -78,9 +78,9 @@
 /*	Arguments:
 /* .IP "\fB-a\fR \fIfamily\fR (default: \fBany\fR)"
 /*	Address family preference: \fBipv4\fR, \fBipv6\fR or \fBany\fR.  When
-/*	using \fBany\fR, \fBposttls-finger\fR(1) will randomly select one of
-/*	the two as the more preferred, and exhaust all MX preferences for the
-/*	first address family before trying any addresses for the other.
+/*	using \fBany\fR, posttls-finger will randomly select one of the two as
+/*	the more preferred, and exhaust all MX preferences for the first
+/*	address family before trying any addresses for the other.
 /* .IP "\fB-A\fR \fItrust-anchor.pem\fR (default: none)"
 /*	A list of PEM trust-anchor files that overrides CAfile and CApath
 /*	trust chain verification.  Specify the option multiple times to
@@ -101,7 +101,7 @@
 /*	fingerprints and matching against user provided certificate
 /*	fingerprints (with DANE TLSA records the algorithm is specified
 /*	in the DNS).  In Postfix versions prior to 3.6, the default value
-/*	was "md5".
+/*	was "sha1".
 /* .IP "\fB-f\fR"
 /*	Lookup the associated DANE TLSA RRset even when a hostname is not an
 /*	alias and its address records lie in an unsigned zone.  See
@@ -111,8 +111,8 @@
 /*	verification.  By default no CAfile is used and no public CAs
 /*	are trusted.
 /* .IP "\fB-g \fIgrade\fR (default: medium)"
-/*	The minimum TLS cipher grade used by \fBposttls-finger\fR(1).
-/*	See smtp_tls_mandatory_ciphers for details.
+/*	The minimum TLS cipher grade used by posttls-finger.  See
+/*	smtp_tls_mandatory_ciphers for details.
 /* .IP "\fB-h \fIhost_lookup\fR (default: \fBdns\fR)"
 /*	The hostname lookup methods used for the connection.  See the
 /*	documentation of smtp_host_lookup for syntax and semantics.
@@ -142,7 +142,7 @@
 /*	security level allows you to test certificate or public-key
 /*	fingerprint matches before you deploy them in the policy table.
 /* .IP
-/*	Note, since \fBposttls-finger\fR(1) does not actually deliver any email,
+/*	Note, since \fBposttls-finger\fR does not actually deliver any email,
 /*	the \fBnone\fR, \fBmay\fR and \fBencrypt\fR security levels are not
 /*	very useful.  Since \fBmay\fR and \fBencrypt\fR don't require peer
 /*	certificates, they will often negotiate anonymous TLS ciphersuites,
@@ -218,14 +218,14 @@
 /*	The TLS policy for MX hosts with "secure" TLSA records when the
 /*	nexthop destination security level is \fBdane\fR, but the MX
 /*	record was found via an "insecure" MX lookup.  See the main.cf
-/*	documentation for smtp_tls_dane_insecure_mx_policy for details.
+/*	documentation for smtp_tls_insecure_mx_policy for details.
 /* .IP "\fB-o \fIname=value\fR"
 /*	Specify zero or more times to override the value of the main.cf
 /*	parameter \fIname\fR with \fIvalue\fR.  Possible use-cases include
 /*	overriding the values of TLS library parameters, or "myhostname" to
 /*	configure the SMTP EHLO name sent to the remote server.
 /* .IP "\fB-p \fIprotocols\fR (default: >=TLSv1)"
-/*	TLS protocols that \fBposttls-finger\fR(1) will exclude or include.  See
+/*	TLS protocols that posttls-finger will exclude or include.  See
 /*	smtp_tls_mandatory_protocols for details.
 /* .IP "\fB-P \fICApath/\fR (default: none)"
 /*	The OpenSSL CApath/ directory (indexed via c_rehash(1)) for remote
@@ -257,10 +257,10 @@
 /*	Enable verbose Postfix logging.  Specify more than once to increase
 /*	the level of verbose logging.
 /* .IP "\fB-w\fR"
-/*	Enable outgoing TLS wrapper mode, or SUBMISSIONS/SMTPS support.  This
-/*	is typically provided on port 465 by servers that are compatible with
-/*	the SMTP-in-SSL protocol, rather than the STARTTLS protocol.
-/*	The destination \fIdomain\fR:\fIport\fR must of course provide such
+/*	Enable outgoing TLS wrapper mode, or SMTPS support.  This is typically
+/*	provided on port 465 by servers that are compatible with the ad-hoc
+/*	SMTP in SSL protocol, rather than the standard STARTTLS protocol.
+/*	The destination \fIdomain\fR:\fIport\fR should of course provide such
 /*	a service.
 /* .IP "\fB-X\fR"
 /*	Enable \fBtlsproxy\fR(8) mode. This is an unsupported mode,
@@ -402,6 +402,7 @@
 
 #ifdef USE_TLS
 #include <tls_proxy.h>
+#include <openssl/engine.h>
 #endif
 
  /*
@@ -617,7 +618,7 @@ static int greeting(STATE *state)
     /*
      * Prepare for disaster.
      */
-    smtp_stream_setup(stream, conn_tmout, /* deadline */ 1, /* minrate */ 0);
+    smtp_stream_setup(stream, conn_tmout, 1);
     if ((except = vstream_setjmp(stream)) != 0) {
 	msg_info("%s while reading server greeting", exception_text(except));
 	return (1);
@@ -652,7 +653,6 @@ static RESPONSE *ehlo(STATE *state)
     /*
      * Send the standard greeting with our hostname
      */
-    smtp_stream_setup(stream, smtp_tmout, /* deadline */ 1, /* minrate */ 0);
     if ((except = vstream_setjmp(stream)) != 0) {
 	msg_info("%s while sending %s", exception_text(except), ehlo);
 	return (0);
@@ -740,7 +740,7 @@ static int starttls(STATE *state)
 
     if (state->wrapper_mode == 0) {
 	/* SMTP stream with deadline timeouts */
-	smtp_stream_setup(stream, smtp_tmout, /* deadline */ 1, /* minrate */ 0);
+	smtp_stream_setup(stream, smtp_tmout, 1);
 	if ((except = vstream_setjmp(stream)) != 0) {
 	    msg_fatal("%s while sending STARTTLS", exception_text(except));
 	    return (1);
@@ -780,7 +780,6 @@ static int starttls(STATE *state)
     else
 	ADD_EXCLUDE(cipher_exclusions, "eNULL");
 
-    smtp_stream_setup(stream, smtp_tmout, /* deadline */ 1, /* minrate */ 0);
     if (state->tlsproxy_mode) {
 	TLS_CLIENT_PARAMS tls_params;
 
@@ -940,9 +939,9 @@ static int starttls(STATE *state)
 		print_trust_info(state);
 	    state->log_mask &= ~(TLS_LOG_CERTMATCH | TLS_LOG_PEERCERT |
 				 TLS_LOG_VERBOSE | TLS_LOG_UNTRUSTED);
+	    state->log_mask |= TLS_LOG_CACHE | TLS_LOG_SUMMARY;
+	    tls_update_app_logmask(state->tls_ctx, state->log_mask);
 	}
-	state->log_mask |= TLS_LOG_CACHE | TLS_LOG_SUMMARY;
-	tls_update_app_logmask(state->tls_ctx, state->log_mask);
     }
     return (0);
 }
@@ -986,7 +985,7 @@ static int doproto(STATE *state)
     /*
      * Prepare for disaster.
      */
-    smtp_stream_setup(stream, smtp_tmout, /* deadline */ 1, /* minrate */ 0);
+    smtp_stream_setup(stream, smtp_tmout, 1);
     if ((except = vstream_setjmp(stream)) != 0) {
 	msg_warn("%s while sending QUIT command", exception_text(except));
 	return (0);
@@ -1251,8 +1250,6 @@ static DNS_RR *addr_one(STATE *state, DNS_RR *addr_list, const char *host,
 		    msg_fatal("host %s: conversion error for address family %d: %m",
 		    host, ((struct sockaddr *) (res0->ai_addr))->sa_family);
 		addr_list = dns_rr_append(addr_list, addr);
-		if (DNS_RR_IS_TRUNCATED(addr_list))
-		    break;
 	    }
 	    freeaddrinfo(res0);
 	    if (found == 0) {
@@ -1290,8 +1287,6 @@ static DNS_RR *mx_addr_list(STATE *state, DNS_RR *mx_names)
 	    msg_panic("%s: bad resource type: %d", myname, rr->type);
 	addr_list = addr_one(state, addr_list, (char *) rr->data, res_opt,
 			     rr->pref);
-	if (addr_list && DNS_RR_IS_TRUNCATED(addr_list))
-	    break;
     }
     return (addr_list);
 }
@@ -1396,7 +1391,7 @@ static DNS_RR *host_addr(STATE *state, const char *host)
     return (addr_list);
 }
 
-/* dane_host_level - candidate host "dane" or degraded security level */
+/* dane_host_level - canidate host "dane" or degraded security level */
 
 static int dane_host_level(STATE *state, DNS_RR *addr)
 {
@@ -1493,14 +1488,12 @@ static char *parse_destination(char *destination, char *def_service,
     /*
      * Convert service to port number, network byte order.
      */
-    service = (char *) filter_known_tcp_port(service);
     if (alldig(service)) {
 	if ((port = atoi(service)) >= 65536 || port == 0)
-	    msg_fatal("bad network port: %s for destination: %s",
-		      service, destination);
+	    msg_fatal("bad network port in destination: %s", destination);
 	*portp = htons(port);
     } else {
-	if ((sp = getservbyname(service, protocol)) != 0)
+	if ((sp = getservbyname(filter_known_tcp_port(service), protocol)) != 0)
 	    *portp = sp->s_port;
 	else if (strcmp(service, "smtp") == 0)
 	    *portp = htons(25);
@@ -2027,19 +2020,7 @@ static void parse_match(STATE *state, int argc, char *argv[])
 #ifdef USE_TLS
     int     smtp_mode = 1;
 
-    /*
-     * DANE match names are configured late, once the TLSA records are in
-     * hand. For now, prepare to fall back to "secure".
-     */
     switch (state->level) {
-    default:
-	state->match = 0;
-	if (*argv)
-	    msg_warn("TLS level '%s' does not implement certificate matching",
-		     str_tls_level(state->level));
-	break;
-    case TLS_LEV_DANE:
-    case TLS_LEV_DANE_ONLY:
     case TLS_LEV_SECURE:
 	state->match = argv_alloc(2);
 	while (*argv)
@@ -2060,6 +2041,11 @@ static void parse_match(STATE *state, int argc, char *argv[])
 	    tls_dane_add_fpt_digests((TLS_DANE *) state->dane, *argv++, "",
 				     smtp_mode);
 	break;
+    case TLS_LEV_DANE:
+    case TLS_LEV_DANE_ONLY:
+	state->match = argv_alloc(2);
+	argv_add(state->match, "nexthop", "hostname", ARGV_END);
+	break;
     }
 #endif
 }
